package javax.net.ssl;

import java.security.InvalidAlgorithmParameterException;
import java.security.KeyStoreException;
import java.security.KeyStore;
import java.security.NoSuchProviderException;
import java.security.NoSuchAlgorithmException;
import sun.security.jca.GetInstance;
import java.security.AccessController;
import java.security.Security;
import java.security.PrivilegedAction;
import java.security.Provider;

public class TrustManagerFactory
{
    private Provider provider;
    private TrustManagerFactorySpi factorySpi;
    private String algorithm;
    
    public static final String getDefaultAlgorithm() {
        String s = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            @Override
            public String run() {
                return Security.getProperty("ssl.TrustManagerFactory.algorithm");
            }
        });
        if (s == null) {
            s = "SunX509";
            return s;
        }
        return s;
    }
    
    protected TrustManagerFactory(TrustManagerFactorySpi factorySpi, Provider provider, String algorithm) {
        this.factorySpi = factorySpi;
        this.provider = provider;
        this.algorithm = algorithm;
    }
    
    public final String getAlgorithm() {
        return this.algorithm;
    }
    
    public static final TrustManagerFactory getInstance(String algorithm) throws NoSuchAlgorithmException {
        GetInstance.Instance instance = GetInstance.getInstance("TrustManagerFactory", TrustManagerFactorySpi.class, algorithm);
        return new TrustManagerFactory((TrustManagerFactorySpi)instance.impl, instance.provider, algorithm);
    }
    
    public static final TrustManagerFactory getInstance(String algorithm, String provider) throws NoSuchAlgorithmException, NoSuchProviderException {
        GetInstance.Instance instance = GetInstance.getInstance("TrustManagerFactory", TrustManagerFactorySpi.class, algorithm, provider);
        return new TrustManagerFactory((TrustManagerFactorySpi)instance.impl, instance.provider, algorithm);
    }
    
    public static final TrustManagerFactory getInstance(String algorithm, Provider provider) throws NoSuchAlgorithmException {
        GetInstance.Instance instance = GetInstance.getInstance("TrustManagerFactory", TrustManagerFactorySpi.class, algorithm, provider);
        return new TrustManagerFactory((TrustManagerFactorySpi)instance.impl, instance.provider, algorithm);
    }
    
    public final Provider getProvider() {
        return this.provider;
    }
    
    public final void init(KeyStore keyStore) throws KeyStoreException {
        this.factorySpi.engineInit(keyStore);
    }
    
    public final void init(ManagerFactoryParameters managerFactoryParameters) throws InvalidAlgorithmParameterException {
        this.factorySpi.engineInit(managerFactoryParameters);
    }
    
    public final TrustManager[] getTrustManagers() {
        return this.factorySpi.engineGetTrustManagers();
    }
}
